//
//  FlutterIcuSegmentviewController.m
//  Colours
//
//  Created by zhuyuhui on 2020/10/10.
//

#import "FlutterIcuSegmentviewController.h"
#import <JXCategoryView/JXCategoryView.h>
#import <Colours/Colours.h>

@interface FlutterIcuSegmentviewController()<JXCategoryViewDelegate>

@end

@implementation FlutterIcuSegmentviewController{
    int64_t _viewId;
    FlutterMethodChannel* _channel;
    JXCategoryTitleView * _categoryView;
    JXCategoryIndicatorLineView *_lineView;
}

- (instancetype)initWithWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger{
    if ([super init]) {
        _viewId = viewId;
        
        _categoryView = [[JXCategoryTitleView alloc] initWithFrame:frame];
        _categoryView.delegate = self;
        _categoryView.titleColorGradientEnabled = YES;
        
        _lineView = [[JXCategoryIndicatorLineView alloc] init];
        _lineView.indicatorWidth = JXCategoryViewAutomaticDimension;
        _categoryView.indicators = @[_lineView];
        
            
        NSString* channelName = [NSString stringWithFormat:@"plugins/flutter_icu_segmentview_%lld", viewId];
        _channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
        __weak __typeof__(self) weakSelf = self;
        [_channel setMethodCallHandler:^(FlutterMethodCall *  call, FlutterResult  result) {
            [weakSelf onMethodCall:call result:result];
        }];
        
        [self initConfigWithArguments:args];
//        NSArray <NSString *>*titles = args[@"titles"];
//        NSString *titleColor = args[@"titleColor"];
//        NSString *titleSelectedColor = args[@"titleSelectedColor"];
//        CGFloat titleFont = [args[@"titleFont"] floatValue];
//        CGFloat titleSelectedFont = [args[@"titleSelectedFont"] floatValue];
//        NSString *indicatorColor = args[@"indicatorColor"];
//
//        NSInteger defaultSelectedIndex = [args[@"defaultSelectedIndex"] integerValue];
//
//        ///数据源设置
//        if (titles) _categoryView.titles = titles;
//        ///默认选中
//        if (defaultSelectedIndex) _categoryView.defaultSelectedIndex = defaultSelectedIndex;
//        ///外观设置
//        if (titleColor) _categoryView.titleColor = [UIColor colorFromHexString:titleColor];
//        if (titleSelectedColor) _categoryView.titleSelectedColor = [UIColor colorFromHexString:titleSelectedColor];
//        if (titleFont) _categoryView.titleFont = [UIFont systemFontOfSize:titleFont];
//        if (titleSelectedFont) _categoryView.titleSelectedFont = [UIFont systemFontOfSize:titleSelectedFont];
//
//        ///指示器外观设置
//        if (indicatorColor) _lineView.indicatorColor = [UIColor colorFromHexString:indicatorColor];
//
//        if (defaultSelectedIndex) {
//            [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(defaultSelectedIndex)}];
//        }
    }
    
    return self;
}

-(UIView *)view{
    return _categoryView;
}

-(void)onMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result{
    if ([[call method] isEqualToString:@"updateTitles"]) {
        NSDictionary *dic = call.arguments;
        if (dic && [dic isKindOfClass:[NSDictionary class]]) {
            NSArray <NSString *>*titles = dic[@"titles"];
            _categoryView.titles = titles;
            [_categoryView reloadData];
        }
    }
    else if ([[call method] isEqualToString:@"updateAppearance"]) {
        NSDictionary *dic = call.arguments;
        if (dic && [dic isKindOfClass:[NSDictionary class]]) {
            NSString *titleColor = dic[@"titleColor"];
            NSString *titleSelectedColor = dic[@"titleSelectedColor"];
            CGFloat titleFont = [dic[@"titleFont"] floatValue];
            CGFloat titleSelectedFont = [dic[@"titleSelectedFont"] floatValue];
            NSString *indicatorColor = dic[@"indicatorColor"];

            ///外观设置
            if (titleColor) _categoryView.titleColor = [UIColor colorFromHexString:titleColor];
            if (titleSelectedColor) _categoryView.titleSelectedColor = [UIColor colorFromHexString:titleSelectedColor];
            if (titleFont) _categoryView.titleFont = [UIFont systemFontOfSize:titleFont];
            if (titleSelectedFont) _categoryView.titleSelectedFont = [UIFont systemFontOfSize:titleSelectedFont];

            ///指示器外观设置
            if (indicatorColor) _lineView.indicatorColor = [UIColor colorFromHexString:indicatorColor];
        }
    }
    else if ([[call method] isEqualToString:@"updateSelectIndex"]) {
        NSDictionary *dic = call.arguments;
        if (dic && [dic isKindOfClass:[NSDictionary class]]) {
            NSInteger selectedIndex = [dic[@"selectIndex"] integerValue];
            [_categoryView selectItemAtIndex:selectedIndex];
            [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(selectedIndex)}];
        }
    }
    else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)initConfigWithArguments:(NSDictionary *)arguments{
    NSArray <NSString *>*titles = arguments[@"titles"];
    NSString *titleColor = arguments[@"titleColor"];
    NSString *titleSelectedColor = arguments[@"titleSelectedColor"];
    CGFloat titleFont = [arguments[@"titleFont"] floatValue];
    CGFloat titleSelectedFont = [arguments[@"titleSelectedFont"] floatValue];
    NSString *indicatorColor = arguments[@"indicatorColor"];

    NSInteger defaultSelectedIndex = [arguments[@"defaultSelectedIndex"] integerValue];

    ///数据源设置
    if (titles) _categoryView.titles = titles;
    ///默认选中
    if (defaultSelectedIndex) _categoryView.defaultSelectedIndex = defaultSelectedIndex;
    ///外观设置
    if (titleColor) _categoryView.titleColor = [UIColor colorFromHexString:titleColor];
    if (titleSelectedColor) _categoryView.titleSelectedColor = [UIColor colorFromHexString:titleSelectedColor];
    if (titleFont) _categoryView.titleFont = [UIFont systemFontOfSize:titleFont];
    if (titleSelectedFont) _categoryView.titleSelectedFont = [UIFont systemFontOfSize:titleSelectedFont];

    ///指示器外观设置
    if (indicatorColor) _lineView.indicatorColor = [UIColor colorFromHexString:indicatorColor];
    
    if (defaultSelectedIndex) {
        [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(defaultSelectedIndex)}];
    }
}



#pragma mark - JXCategoryViewDelegate

- (void)categoryView:(JXCategoryBaseView *)categoryView didClickSelectedItemAtIndex:(NSInteger)index {
    [_channel invokeMethod:@"didClickSelectedItemAtIndex" arguments:@{@"index" : @(index)}];
}




@end



@implementation FlutterIcuSegmentViewFactory{
    NSObject<FlutterBinaryMessenger>*_messenger;
}

- (instancetype)initWithMessenger:(NSObject<FlutterBinaryMessenger> *)messager{
    self = [super init];
    if (self) {
        _messenger = messager;
    }
    return self;
}

-(NSObject<FlutterMessageCodec> *)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

-(NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args{
    
   FlutterIcuSegmentviewController*activity = [[FlutterIcuSegmentviewController alloc] initWithWithFrame:frame viewIdentifier:viewId arguments:args binaryMessenger:_messenger];
    return activity;
}
@end
