import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

///先定义一个controller创建成功回调 后面会用到
typedef void ViewCreatedCallback(MethodChannelSegmentViewPlatform controller);

typedef void DidClickSelectedItemAtIndex(int index);

class MethodChannelSegmentViewPlatform {
  final _PlatformCallbacksHandler _platformCallbacksHandler;
  final MethodChannel _channel;

  MethodChannelSegmentViewPlatform(int id, this._platformCallbacksHandler)
      : assert(_platformCallbacksHandler != null),
        _channel = MethodChannel('plugins/flutter_icu_segmentview_$id') {
    _channel.setMethodCallHandler(_onMethodCall);
  }

  Future<void> updateTitles(List titles) async {
    return _channel.invokeMethod('updateTitles', {
      "titles": titles,
    });
  }

  Future<void> updateAppearance({
    String titleColor = '0xFF000000',
    String titleSelectedColor = '0xFFDC143C',
    double titleFont = 14,
    double titleSelectedFont = 15,
    String indicatorColor = '0xFFDC143C',
  }) async {
    return _channel.invokeMethod('updateAppearance', {
      "titleColor": titleColor,
      "titleSelectedColor": titleSelectedColor,
      "titleFont": titleFont,
      "titleSelectedFont": titleSelectedFont,
      "indicatorColor": indicatorColor,
    });
  }

  Future<void> updateSelectIndex(int selectedIndex) async {
    return _channel.invokeMethod('updateSelectIndex', {
      "selectIndex": selectedIndex,
    });
  }

  Future<bool> _onMethodCall(MethodCall call) async {
    switch (call.method) {
      case 'didClickSelectedItemAtIndex':
        final int index = call.arguments['index'];
        _platformCallbacksHandler.didClickSelectedItemAtIndex(index);
        return true;
    }
    throw MissingPluginException(
      '${call.method} was invoked but has no handler',
    );
  }
}

class FlutterIcuSegmentview extends StatefulWidget {
  final ViewCreatedCallback onViewCreated;
  final DidClickSelectedItemAtIndex didClickSelectedItemAtIndex;

  final Map<String, dynamic> arguments;
  const FlutterIcuSegmentview(
      {Key key,
      this.onViewCreated,
      this.didClickSelectedItemAtIndex,
      this.arguments})
      : super(key: key);

  @override
  _FlutterIcuSegmentviewState createState() => _FlutterIcuSegmentviewState();
}

class _FlutterIcuSegmentviewState extends State<FlutterIcuSegmentview> {
  _PlatformCallbacksHandler _platformCallbacksHandler;

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return UiKitView(
        viewType: "plugins/flutter_icu_segmentview",
        onPlatformViewCreated: _onPlatformViewCreated,
        creationParams: widget.arguments,
        creationParamsCodec: StandardMessageCodec(),
      );
    }
    return Text('不支持的平台 $defaultTargetPlatform');
  }

  void _onPlatformViewCreated(int id) {
    if (widget.onViewCreated == null) {
      return;
    }
    widget.onViewCreated(
        MethodChannelSegmentViewPlatform(id, _platformCallbacksHandler));
  }

  @override
  void initState() {
    super.initState();
    _platformCallbacksHandler = _PlatformCallbacksHandler(widget);
  }

  @override
  void didUpdateWidget(FlutterIcuSegmentview oldWidget) {
    super.didUpdateWidget(oldWidget);
    _platformCallbacksHandler._widget = widget;
  }
}

class _PlatformCallbacksHandler {
  FlutterIcuSegmentview _widget;

  _PlatformCallbacksHandler(this._widget);

  void didClickSelectedItemAtIndex(int index) {
    if (_widget.didClickSelectedItemAtIndex != null) {
      _widget.didClickSelectedItemAtIndex(index);
    }
  }
}
